@@ -27,6 +27,8 @@
 include("./include/auth.php");
 include_once("./lib/data_query.php");
 include_once("./lib/utility.php");
+include_once("./lib/sort.php");
+include_once("./lib/html_form_template.php");
 include_once("./lib/template.php");
 
 /* set default action */
@@ -35,18 +37,18 @@ if (!isset($_REQUEST["action"])) { $_REQUEST["action"] = ""; }
 switch ($_REQUEST["action"]) {
 	case 'save':
 		form_save();
-		
+
 		break;
 	case 'query_reload':
 		host_reload_query();
-		
+
 		header("Location: graphs_new.php?host_id=" . $_GET["host_id"]);
 		break;
 	default:
 		include_once("./include/top_header.php");
-		
+
 		graphs();
-		
+
 		include_once("./include/bottom_footer.php");
 		break;
 }
@@ -69,18 +71,18 @@ function form_save() {
 				$selected_graphs["sg"]{$matches[1]}{$_POST{"sgg_" . $matches[1]}}{$matches[2]} = true;
 			}
 		}
-		
+
 		if (isset($selected_graphs)) {
 			host_new_graphs($_POST["host_id"], $_POST["host_template_id"], $selected_graphs);
 			exit;
 		}
-		
+
 		header("Location: graphs_new.php?host_id=" . $_POST["host_id"]);
 	}
-	
+
 	if (isset($_POST["save_component_new_graphs"])) {
 		host_new_graphs_save();
-		
+
 		header("Location: graphs_new.php?host_id=" . $_POST["host_id"]);
 	}
 }
@@ -91,7 +93,7 @@ function form_save() {
 
 function draw_edit_form_row($field_array, $field_name, $previous_value) {
 	$field_array["value"] = $previous_value;
-	
+
 	draw_edit_form(
 		array(
 			"config" => array(
@@ -110,7 +112,7 @@ function draw_edit_form_row($field_array, $field_name, $previous_value) {
    ------------------- */
 
 function host_reload_query() {
-	data_query($_GET["host_id"], $_GET["id"]);
+	run_data_query($_GET["host_id"], $_GET["id"]);
 }
 
 /* -------------------
@@ -118,10 +120,8 @@ function host_reload_query() {
    ------------------- */
 
 function host_new_graphs_save() {
-	global $struct_graph, $struct_data_source, $struct_graph_item, $struct_data_source_item, $paths;
-	
 	$selected_graphs_array = unserialize(stripslashes($_POST["selected_graphs_array"]));
-	
+
 	/* form an array that contains all of the data on the previous form */
 	while (list($var, $val) = each($_POST)) {
 		if (preg_match("/^g_(\d+)_(\d+)_(\w+)/", $var, $matches)) { /* 1: snmp_query_id, 2: graph_template_id, 3: field_name */
@@ -136,7 +136,7 @@ function host_new_graphs_save() {
 				graph_template_item_id
 				from graph_template_input_defs
 				where graph_template_input_id=" . $matches[3]);
-			
+
 			/* loop through each item affected and update column data */
 			if (sizeof($item_list) > 0) {
 			foreach ($item_list as $item) {
@@ -167,10 +167,12 @@ function host_new_graphs_save() {
 			}
 		}
 	}
-	
+
+	debug_log_clear("new_graphs");
+
 	while (list($form_type, $form_array) = each($selected_graphs_array)) {
 		$current_form_type = $form_type;
-		
+
 		while (list($form_id1, $form_array2) = each($form_array)) {
 			/* enumerate information from the arrays stored in post variables */
 			if ($form_type == "cg") {
@@ -178,131 +180,74 @@ function host_new_graphs_save() {
 			}elseif ($form_type == "sg") {
 				while (list($form_id2, $form_array3) = each($form_array2)) {
 					$snmp_index_array = $form_array3;
-					
+
 					$snmp_query_array["snmp_query_id"] = $form_id1;
-					$snmp_query_array["snmp_index_on"] = $_POST{"sg_" . $form_id1};
+					$snmp_query_array["snmp_index_on"] = get_best_data_query_index_type($_POST["host_id"], $form_id1);
 					$snmp_query_array["snmp_query_graph_id"] = $form_id2;
 				}
-				
+
 				$graph_template_id = db_fetch_cell("select graph_template_id from snmp_query_graph where id=" . $snmp_query_array["snmp_query_graph_id"]);
 			}
-			
+
 			if ($current_form_type == "cg") {
 				$return_array = create_complete_graph_from_template($graph_template_id, $_POST["host_id"], "", $values["cg"]);
+
+				debug_log_insert("new_graphs", "Created graph: " . get_graph_title($return_array["local_graph_id"]));
 			}elseif ($current_form_type == "sg") {
 				while (list($snmp_index, $true) = each($snmp_index_array)) {
-					$snmp_query_array["snmp_index"] = decode_data_query_index($snmp_index, $snmp_query_array["snmp_query_id"], $_POST["host_id"]); 
-					
+					$snmp_query_array["snmp_index"] = decode_data_query_index($snmp_index, $snmp_query_array["snmp_query_id"], $_POST["host_id"]);
+
 					$return_array = create_complete_graph_from_template($graph_template_id, $_POST["host_id"], $snmp_query_array, $values["sg"]{$snmp_query_array["snmp_query_id"]});
+
+					debug_log_insert("new_graphs", "Created graph: " . get_graph_title($return_array["local_graph_id"]));
 				}
 			}
 		}
 	}
-	
+
 	/* lastly push host-specific information to our data sources */
 	push_out_host($_POST["host_id"],0);
 }
 
 function host_new_graphs($host_id, $host_template_id, $selected_graphs_array) {
+	global $colors;
+
+	/* we use object buffering on this page to allow redirection to another page if no
+	fields are actually drawn */
+	ob_start();
+
 	include_once("./include/top_header.php");
-	
-	global $colors, $paths, $struct_graph, $row_counter, $struct_data_source, $struct_graph_item, $struct_data_source_item;
-	
+
 	print "<form method='post' action='graphs_new.php'>\n";
-	
+
 	$snmp_query_id = 0;
+	$num_output_fields = array();
+
 	while (list($form_type, $form_array) = each($selected_graphs_array)) {
 		while (list($form_id1, $form_array2) = each($form_array)) {
 			if ($form_type == "cg") {
 				$graph_template_id = $form_id1;
-				
-				start_box("<strong>Create 1 Graph from Host Template", "98%", $colors["header"], "3", "center", "");
+
+				html_start_box("<strong>Create Graph from '" . db_fetch_cell("select name from graph_templates where id=$graph_template_id") . "'", "98%", $colors["header"], "3", "center", "");
 			}elseif ($form_type == "sg") {
 				while (list($form_id2, $form_array3) = each($form_array2)) {
 					$snmp_query_id = $form_id1;
 					$snmp_query_graph_id = $form_id2;
 					$num_graphs = sizeof($form_array3);
-					
+
 					$snmp_query = db_fetch_row("select
 						snmp_query.name,
 						snmp_query.xml_path
 						from snmp_query
 						where snmp_query.id=$snmp_query_id");
-					
+
 					$graph_template_id = db_fetch_cell("select graph_template_id from snmp_query_graph where id=$snmp_query_graph_id");
-					
-					/* list each index that was selected */
-					$i = 0;
-					while (list($snmp_index, $one) = each($form_array3)) {
-						$snmp_indexes[$snmp_query_id][$i] = decode_data_query_index($snmp_index, $snmp_query_id, $host_id); 
-						
-						$i++;
-					}
 				}
-				
+
 				/* DRAW: Data Query */
-				start_box("<strong>Create $num_graphs Graph" . (($num_graphs>1) ? "s" : "") . " from Data Query", "98%", $colors["header"], "3", "center", "");
-				
-				print "<tr><td colspan='2' bgcolor='#" . $colors["header_panel"] . "'><span style='font-size: 10px; color: white;'><strong>Data Query</strong> [" . $snmp_query["name"] . "]</span></td></tr>";
-				
-				$fields = db_fetch_assoc("select
-					data_template.id as data_template_id,
-					data_template.name,
-					data_input_fields.name as field_name,
-					data_input_fields.id as data_input_field_id
-					from snmp_query_field,data_input_fields,data_template,snmp_query_graph,snmp_query_graph_rrd
-					where snmp_query_field.data_input_field_id=data_input_fields.id
-					and snmp_query_field.snmp_query_id=snmp_query_graph.snmp_query_id
-					and snmp_query_graph.id=snmp_query_graph_rrd.snmp_query_graph_id
-					and snmp_query_graph_rrd.data_template_id=data_template.id
-					and snmp_query_field.snmp_query_id=$snmp_query_id
-					and snmp_query_graph_rrd.snmp_query_graph_id=$snmp_query_graph_id
-					and snmp_query_field.action_id=1
-					group by snmp_query_graph_rrd.data_template_id");
-				
-				if (sizeof($fields) > 0) {
-				foreach ($fields as $field) {
-					print "	<tr bgcolor='#" . $colors["form_alternate1"] . "'>
-							<td width='50%'>
-								<font class='textEditTitle'>" . $field["name"] . " -> " . $field["field_name"] . "</font><br>
-								The data for each new graph will be indexed off of the following field. Certain fields may
-								have been removed from the list if they do not contain unique data. It is best to choose a field whose
-								value never changes.
-							</td>";
-					
-					$snmp_queries = get_data_query_array($snmp_query_id);
-					
-					$xml_outputs = array();
-					
-					/* create an SQL string that contains each index in this snmp_index_id */
-					$sql_or = array_to_sql_or($snmp_indexes[$snmp_query_id], "snmp_index");
-					
-					/* list each of the input fields for this snmp query */
-					while (list($field_name, $field_array) = each($snmp_queries["fields"])) {
-						if ($field_array["direction"] == "input") {
-							/* create a list of all values for this index */
-							$field_values = db_fetch_assoc("select field_value from host_snmp_cache where host_id=$host_id and snmp_query_id=$snmp_query_id and field_name='$field_name' and $sql_or");
-							
-							/* aggregate the above list so there is no duplicates */
-							$aggregate_field_values = array_rekey($field_values, "field_value", "field_value");
-							
-							/* fields that contain duplicate or empty values are not suitable to index off of */
-							if (!((sizeof($aggregate_field_values) < sizeof($field_values)) || (in_array("", $aggregate_field_values) == true) || (sizeof($aggregate_field_values) == 0))) {
-								$xml_outputs[$field_name] = $field_name . " (" . $field_array["name"] . ")";
-							}
-						}
-					}
-					
-					print "<td>";
-					form_dropdown("sg_" . $snmp_query_id,$xml_outputs,"","","","","");
-					print "</td>";
-					
-					print "</tr>\n";
-				}
-				}else{ print "<tr bgcolor='#" . $colors["form_alternate1"] . "'><td colspan='2' style='font-weight: bold; color: red;'>There appears to be a problem with your data query. Please make sure you have made at least one data template association.</td></tr>";
-				}
+				html_start_box("<strong>Create $num_graphs Graph" . (($num_graphs>1) ? "s" : "") . " from '" . db_fetch_cell("select name from snmp_query where id=$snmp_query_id") . "'", "98%", $colors["header"], "3", "center", "");
 			}
-			
+
 			$data_templates = db_fetch_assoc("select
 				data_template.name as data_template_name,
 				data_template_rrd.data_source_name,
@@ -325,37 +270,57 @@ function host_new_graphs($host_id, $host_template_id, $selected_graphs_array) {
 				and graph_templates.id=" . $graph_template_id . "
 				and graph_templates_graph.local_graph_id=0");
 			$graph_template_name = db_fetch_cell("select name from graph_templates where id=" . $graph_template_id);
-			
-			draw_nontemplated_fields_graph($graph_template_id, $graph_template, "g_$snmp_query_id" . "_" . $graph_template_id . "_|field|", "<strong>Graph</strong> [Template: " . $graph_template["graph_template_name"] . "]", false, false, (isset($snmp_query_graph_id) ? $snmp_query_graph_id : 0));
-			draw_nontemplated_fields_graph_item($graph_template_id, 0, "gi_" . $snmp_query_id . "_" . $graph_template_id . "_|id|_|field|", "<strong>Graph Items</strong> [Template: " . $graph_template_name . "]", false);
-			
+
+			array_push($num_output_fields, draw_nontemplated_fields_graph($graph_template_id, $graph_template, "g_$snmp_query_id" . "_" . $graph_template_id . "_|field|", "<strong>Graph</strong> [Template: " . $graph_template["graph_template_name"] . "]", false, false, (isset($snmp_query_graph_id) ? $snmp_query_graph_id : 0)));
+			array_push($num_output_fields, draw_nontemplated_fields_graph_item($graph_template_id, 0, "gi_" . $snmp_query_id . "_" . $graph_template_id . "_|id|_|field|", "<strong>Graph Items</strong> [Template: " . $graph_template_name . "]", false));
+
 			/* DRAW: Data Sources */
 			if (sizeof($data_templates) > 0) {
 			foreach ($data_templates as $data_template) {
-				draw_nontemplated_fields_data_source($data_template["data_template_id"], 0, $data_template, "d_" . $snmp_query_id . "_" . $graph_template_id . "_" . $data_template["data_template_id"] . "_|field|", "<strong>Data Source</strong> [Template: " . $data_template["data_template_name"] . "]", false, false, (isset($snmp_query_graph_id) ? $snmp_query_graph_id : 0));
-				
+				array_push($num_output_fields, draw_nontemplated_fields_data_source($data_template["data_template_id"], 0, $data_template, "d_" . $snmp_query_id . "_" . $graph_template_id . "_" . $data_template["data_template_id"] . "_|field|", "<strong>Data Source</strong> [Template: " . $data_template["data_template_name"] . "]", false, false, (isset($snmp_query_graph_id) ? $snmp_query_graph_id : 0)));
+
 				$data_template_items = db_fetch_assoc("select
 					data_template_rrd.*
 					from data_template_rrd
 					where data_template_rrd.data_template_id=" . $data_template["data_template_id"] . "
 					and local_data_id=0");
-				
-				draw_nontemplated_fields_data_source_item($data_template["data_template_id"], $data_template_items, "di_" . $snmp_query_id . "_" . $graph_template_id . "_" . $data_template["data_template_id"] . "_|id|_|field|", "", false, false, false, (isset($snmp_query_graph_id) ? $snmp_query_graph_id : 0));
-				draw_nontemplated_fields_custom_data($data_template["id"], "c_" . $snmp_query_id . "_" . $graph_template_id . "_" . $data_template["data_template_id"] . "_|id|", "<strong>Custom Data</strong> [Template: " . $data_template["data_template_name"] . "]", false, false, $snmp_query_id);
+
+				array_push($num_output_fields, draw_nontemplated_fields_data_source_item($data_template["data_template_id"], $data_template_items, "di_" . $snmp_query_id . "_" . $graph_template_id . "_" . $data_template["data_template_id"] . "_|id|_|field|", "", false, false, false, (isset($snmp_query_graph_id) ? $snmp_query_graph_id : 0)));
+				array_push($num_output_fields, draw_nontemplated_fields_custom_data($data_template["id"], "c_" . $snmp_query_id . "_" . $graph_template_id . "_" . $data_template["data_template_id"] . "_|id|", "<strong>Custom Data</strong> [Template: " . $data_template["data_template_name"] . "]", false, false, $snmp_query_id));
 			}
 			}
-			
-			end_box();
+
+			html_end_box();
 		}
 	}
-	
-	form_hidden_id("host_template_id",$host_template_id);
-	form_hidden_id("host_id",$host_id);
-	form_hidden_box("save_component_new_graphs","1","");
+
+	/* no fields were actually drawn on the form; just save without prompting the user */
+	if (array_sum($num_output_fields) == 0) {
+		ob_end_clean();
+
+		/* since the user didn't actually click "Create" to POST the data; we have to
+		pretend like they did here */
+		$_POST["host_template_id"] = $host_template_id;
+		$_POST["host_id"] = $host_id;
+		$_POST["save_component_new_graphs"] = "1";
+		$_POST["selected_graphs_array"] = serialize($selected_graphs_array);
+
+		host_new_graphs_save();
+
+		header("Location: graphs_new.php?host_id=" . $_POST["host_id"]);
+		exit;
+	}
+
+	/* flush the current output buffer to the browser */
+	ob_end_flush();
+
+	form_hidden_box("host_template_id", $host_template_id, "0");
+	form_hidden_box("host_id", $host_id, "0");
+	form_hidden_box("save_component_new_graphs", "1", "");
 	print "<input type='hidden' name='selected_graphs_array' value='" . serialize($selected_graphs_array) . "'>\n";
-	
-	form_save_button("graphs_new.php?id=$host_id");
-	
+
+	form_save_button("graphs_new.php?host_id=$host_id");
+
 	include_once("./include/bottom_footer.php");
 }
 
@@ -365,18 +330,34 @@ function host_new_graphs($host_id, $host_template_id, $selected_graphs_array) {
 
 function graphs() {
 	global $colors;
-	
+
 	/* use the first host in the list as the default */
 	if ((!isset($_SESSION["sess_graphs_new_host_id"])) && (empty($_REQUEST["host_id"]))) {
 		$_REQUEST["host_id"] = db_fetch_cell("select id from host order by description,hostname limit 1");
 	}
-	
+
 	/* remember these search fields in session vars so we don't have to keep passing them around */
 	if (isset($_REQUEST["host_id"])) { $_SESSION["sess_graphs_new_host_id"] = $_REQUEST["host_id"]; }else{ $_REQUEST["host_id"] = $_SESSION["sess_graphs_new_host_id"]; }
-	
+
 	$host = db_fetch_row("select id,description,hostname,host_template_id from host where id=" . $_REQUEST["host_id"]);
-	
+
+	$debug_log = debug_log_return("new_graphs");
+
+	if (!empty($debug_log)) {
+		debug_log_clear("new_graphs");
+		?>
+		<table width='98%' style='background-color: #f5f5f5; border: 1px solid #bbbbbb;' align='center'>
+			<tr bgcolor="<?php print $colors["light"];?>">
+				<td style="padding: 3px; font-family: monospace;">
+					<?php print $debug_log;?>
+				</td>
+			</tr>
+		</table>
+		<br>
+		<?php
+	}
 	?>
+
 	<form name="form_graph_id">
 	<table width="98%" align="center">
 		<tr>
@@ -395,7 +376,7 @@ function graphs() {
 			<td>
 			</td>
 		</tr>
-	
+
 		<tr>
 			<td class="textArea" style="padding: 3px;" width="300" nowrap>
 				Create new graphs for the following host:
@@ -409,7 +390,7 @@ function graphs() {
 				<select name="cbo_graph_id" onChange="window.location=document.form_graph_id.cbo_graph_id.options[document.form_graph_id.cbo_graph_id.selectedIndex].value">
 					<?php
 					$hosts = db_fetch_assoc("select id,CONCAT_WS('',description,' (',hostname,')') as name from host order by description,hostname");
-					
+
 					if (sizeof($hosts) > 0) {
 					foreach ($hosts as $item) {
 						print "<option value='graphs_new.php?host_id=" . $item["id"] . "'"; if ($_REQUEST["host_id"] == $item["id"]) { print " selected"; } print ">" . $item["name"] . "</option>\n";
@@ -420,27 +401,27 @@ function graphs() {
 			</td>
 		</tr>
 	</table>
-	
-	
+
+
 	</form>
 	<form name="chk" method="post" action="graphs_new.php">
 	<?php
 	$total_rows = sizeof(db_fetch_assoc("select graph_template_id from host_graph where host_id=" . $_REQUEST["host_id"]));
-	
+
 	/* we give users the option to turn off the javascript features for data queries with lots of rows */
 	if (read_config_option("max_data_query_javascript_rows") >= $total_rows) {
 		$use_javascript = true;
 	}else{
 		$use_javascript = false;
 	}
-	
-	start_box("<strong>Graph Templates</strong>", "98%", $colors["header"], "3", "center", "");
-	
+
+	html_start_box("<strong>Graph Templates</strong>", "98%", $colors["header"], "3", "center", "");
+
 	print "	<tr bgcolor='#" . $colors["header_panel"] . "'>
 			<td class='textSubHeaderDark'>Graph Template Name</td>
-			<td width='1%' align='center' bgcolor='#819bc0' style='" . get_checkbox_style() . "'><input type='checkbox' style='margin: 0px;' name='all' title='Select All' onClick='SelectAll(\"cg\");gt_update_selection_indicators();'></td>\n
+			<td width='1%' align='center' bgcolor='#819bc0' style='" . get_checkbox_style() . "'><input type='checkbox' style='margin: 0px;' name='all_cg' title='Select All' onClick='SelectAll(\"cg\",this.checked);gt_update_selection_indicators();'></td>\n
 		</tr>\n";
-	
+
 	$graph_templates = db_fetch_assoc("select
 		graph_templates.id as graph_template_id,
 		graph_templates.name as graph_template_name
@@ -448,34 +429,40 @@ function graphs() {
 		where host_graph.graph_template_id=graph_templates.id
 		and host_graph.host_id=" . $_REQUEST["host_id"] . "
 		order by graph_templates.name");
-	
-	$template_graphs = db_fetch_assoc("select graph_local.graph_template_id from graph_local,host_graph where graph_local.graph_template_id=host_graph.graph_template_id and graph_local.host_id=" . $host["id"] . " group by graph_local.graph_template_id");
-	
+
+	$template_graphs = db_fetch_assoc("select
+		graph_local.graph_template_id
+		from graph_local,host_graph
+		where graph_local.graph_template_id=host_graph.graph_template_id
+		and graph_local.host_id=host_graph.host_id
+		and graph_local.host_id=" . $host["id"] . "
+		group by graph_local.graph_template_id");
+
 	print "<script type='text/javascript'>\nvar gt_created_graphs = new Array()\n</script>\n";
-	
+
 	if ((sizeof($template_graphs) > 0) && ($use_javascript == true)) {
 		print "<script type='text/javascript'>\n<!--\n";
 		print "var gt_created_graphs = new Array(";
-		
+
 		$cg_ctr = 0;
 		foreach ($template_graphs as $template_graph) {
-			print (($cg_ctr > 0) ? "," : "") . "'" . $template_graph["graph_template_id"] . "'"; 
-			
+			print (($cg_ctr > 0) ? "," : "") . "'" . $template_graph["graph_template_id"] . "'";
+
 			$cg_ctr++;
 		}
-		
+
 		print ")\n";
 		print "//-->\n</script>\n";
 	}
-	
+
 	/* create a row for each graph template associated with the host template */
 	$i = 0;
 	if (sizeof($graph_templates) > 0) {
 	foreach ($graph_templates as $graph_template) {
 		$query_row = $graph_template["graph_template_id"];
-		
+
 		print "<tr id='gt_line$query_row' bgcolor='#" . (($i % 2 == 0) ? "ffffff" : $colors["light"]) . "'>"; $i++;
-		
+
 		print "		<td" . (($use_javascript == true) ? " onClick='gt_select_line(" . $graph_template["graph_template_id"] . ");'" : "") . "><span id='gt_text$query_row" . "_0'>
 					<span id='gt_text$query_row" . "_0'><strong>Create:</strong> " . $graph_template["graph_template_name"] . "</span>
 				</td>
@@ -485,11 +472,11 @@ function graphs() {
 			</tr>";
 	}
 	}
-	
+
 	if ($use_javascript == true) {
 		print "<script type='text/javascript'>gt_update_deps(1);</script>\n";
 	}
-	
+
 	/* create a row at the bottom that lets the user create any graph they choose */
 	print "	<tr bgcolor='#" . (($i % 2 == 0) ? "ffffff" : $colors["light"]) . "'>
 			<td colspan='2' width='60' nowrap>
@@ -497,9 +484,9 @@ function graphs() {
 				form_dropdown("cg_g", db_fetch_assoc("select id,name from graph_templates order by name"), "name", "id", "", "(Select a graph type to create)", "", "font-size: 10px;");
 	print "		</td>
 		</tr>";
-	
-	end_box();
-	
+
+	html_end_box();
+
 	$snmp_queries = db_fetch_assoc("select
 		snmp_query.id,
 		snmp_query.name,
@@ -508,73 +495,76 @@ function graphs() {
 		where host_snmp_query.snmp_query_id=snmp_query.id
 		and host_snmp_query.host_id=" . $host["id"] . "
 		order by snmp_query.name");
-	
+
 	print "<script type='text/javascript'>\nvar created_graphs = new Array()\n</script>\n";
-	
+
 	if (sizeof($snmp_queries) > 0) {
 	foreach ($snmp_queries as $snmp_query) {
 		unset($total_rows);
-		
+
 		$xml_array = get_data_query_array($snmp_query["id"]);
-		$xml_outputs = array();
-		
-		$total_rows = 0;
+
 		$num_input_fields = 0;
 		$num_visible_fields = 0;
-		
+
 		if ($xml_array != false) {
 			/* loop through once so we can find out how many input fields there are */
+			reset($xml_array["fields"]);
 			while (list($field_name, $field_array) = each($xml_array["fields"])) {
 				if ($field_array["direction"] == "input") {
 					$num_input_fields++;
-					
+
 					if (!isset($total_rows)) {
-						$total_rows = sizeof(db_fetch_assoc("select snmp_index from host_snmp_cache where host_id=" . $host["id"] . " and snmp_query_id=" . $snmp_query["id"] . " and field_name='$field_name'"));
+						$total_rows = db_fetch_cell("select count(*) from host_snmp_cache where host_id=" . $host["id"] . " and snmp_query_id=" . $snmp_query["id"] . " and field_name='$field_name'");
 					}
 				}
 			}
 		}
-		
+
+		if (!isset($total_rows)) {
+			$total_rows = 0;
+		}
+
 		/* we give users the option to turn off the javascript features for data queries with lots of rows */
 		if (read_config_option("max_data_query_javascript_rows") >= $total_rows) {
 			$use_javascript = true;
 		}else{
 			$use_javascript = false;
 		}
-		
+
 		$snmp_query_graphs = db_fetch_assoc("select snmp_query_graph.id,snmp_query_graph.name from snmp_query_graph where snmp_query_graph.snmp_query_id=" . $snmp_query["id"] . " order by snmp_query_graph.name");
-		
+
 		if ((sizeof($snmp_query_graphs) > 0) && ($use_javascript == true)) {
 			print "<script type='text/javascript'>\n<!--\n";
-			
+
 			foreach ($snmp_query_graphs as $snmp_query_graph) {
-				$created_graphs = db_fetch_assoc("select
+				$created_graphs = db_fetch_assoc("select distinct
 					data_local.snmp_index
 					from data_local,data_template_data
-					left join data_input_data on data_template_data.id=data_input_data.data_template_data_id 
+					left join data_input_data on data_template_data.id=data_input_data.data_template_data_id
 					left join data_input_fields on data_input_data.data_input_field_id=data_input_fields.id
 					where data_local.id=data_template_data.local_data_id
 					and data_input_fields.type_code='output_type'
 					and data_input_data.value='" . $snmp_query_graph["id"] . "'
 					and data_local.host_id=" . $host["id"]);
-				
+
 				print "created_graphs[" . $snmp_query_graph["id"] . "] = new Array(";
-				
+
 				$cg_ctr = 0;
 				if (sizeof($created_graphs) > 0) {
 				foreach ($created_graphs as $created_graph) {
-					print (($cg_ctr > 0) ? "," : "") . "'" . encode_data_query_index($created_graph["snmp_index"]) . "'"; 
-					
+					print (($cg_ctr > 0) ? "," : "") . "'" . encode_data_query_index($created_graph["snmp_index"]) . "'";
+
 					$cg_ctr++;
 				}
 				}
-				
+
 				print ")\n";
 			}
-			
+
 			print "//-->\n</script>\n";
 		}
-		
+
 		print "	<table width='98%' style='background-color: #" . $colors["form_alternate2"] . "; border: 1px solid #" . $colors["header"] . ";' align='center' cellpadding='3' cellspacing='0'>\n
 				<tr>
 					<td bgcolor='#" . $colors["header"] . "' colspan='" . ($num_input_fields+1) . "'>
@@ -590,26 +580,29 @@ function graphs() {
 						</table>
 					</td>
 				</tr>";
-		
+
 		if ($xml_array != false) {
 			$html_dq_header = "";
 			$snmp_query_indexes = array();
-			
+
 			reset($xml_array["fields"]);
 			while (list($field_name, $field_array) = each($xml_array["fields"])) {
 				if ($field_array["direction"] == "input") {
 					$raw_data = db_fetch_assoc("select field_value,snmp_index from host_snmp_cache where host_id=" . $host["id"] . " and field_name='$field_name' and snmp_query_id=" . $snmp_query["id"]);
-					
+
 					/* don't even both to display the column if it has no data */
 					if (sizeof($raw_data) > 0) {
 						/* draw each header item <TD> */
 						$html_dq_header .= "<td height='1'><strong><font color='#" . $colors["header_text"] . "'>" . $field_array["name"] . "</font></strong></td>\n";
-						
+
 						foreach ($raw_data as $data) {
 							$snmp_query_data[$field_name]{$data["snmp_index"]} = $data["field_value"];
-							$snmp_query_indexes{$data["snmp_index"]} = $data["snmp_index"];
+
+							if (!in_array($data["snmp_index"], $snmp_query_indexes)) {
+								array_push($snmp_query_indexes, $data["snmp_index"]);
+							}
 						}
-						
+
 						$num_visible_fields++;
 					}elseif (sizeof($raw_data) == 0) {
 						/* we are choosing to not display this column, so unset the associated
@@ -618,25 +611,34 @@ function graphs() {
 					}
 				}
 			}
-			
+
+			/* if the user specified a prefered sort order; sort the list of indexes before displaying them */
+			if (isset($xml_array["index_order_type"])) {
+				if ($xml_array["index_order_type"] == "numeric") {
+					usort($snmp_query_indexes, "usort_numeric");
+				}else if ($xml_array["index_order_type"] == "alphabetic") {
+					usort($snmp_query_indexes, "usort_alphabetic");
+				}
+			}
+
 			if ($num_visible_fields == 0) {
 				print "<tr bgcolor='#" . $colors["form_alternate1"] . "'><td>This data query returned 0 rows, perhaps there was a problem executing this
-					data query. You can <a href='host.php?action=query_verbose&id=" . $snmp_query["id"] . "&host_id=" . $host["id"] . "'>run this data 
+					data query. You can <a href='host.php?action=query_verbose&id=" . $snmp_query["id"] . "&host_id=" . $host["id"] . "'>run this data
 					query in debug mode</a> to get more information.</td></tr>\n";
 			}else{
 				print "	<tr bgcolor='#" . $colors["header_panel"] . "'>
 						$html_dq_header
-						<td width='1%' align='center' bgcolor='#819bc0' style='" . get_checkbox_style() . "'><input type='checkbox' style='margin: 0px;' name='all' title='Select All' onClick='SelectAll(\"sg_" . $snmp_query["id"] . "\");" . (($use_javascript == true) ? "dq_update_selection_indicators();" : "") . "'></td>\n
+						<td width='1%' align='center' bgcolor='#819bc0' style='" . get_checkbox_style() . "'><input type='checkbox' style='margin: 0px;' name='all_" . $snmp_query["id"] . "' title='Select All' onClick='SelectAll(\"sg_" . $snmp_query["id"] . "\",this.checked);" . (($use_javascript == true) ? "dq_update_selection_indicators();" : "") . "'></td>\n
 					</tr>\n";
 			}
-			
+
 			$row_counter = 0;
 			if (sizeof($snmp_query_indexes) > 0) {
-			while (list($snmp_index, $snmp_index) = each($snmp_query_indexes)) {
+			while (list($id, $snmp_index) = each($snmp_query_indexes)) {
 				$query_row = $snmp_query["id"] . "_" . encode_data_query_index($snmp_index);
-				
+
 				print "<tr id='line$query_row' bgcolor='#" . (($row_counter % 2 == 0) ? "ffffff" : $colors["light"]) . "'>"; $i++;
-				
+
 				$column_counter = 0;
 				reset($xml_array["fields"]);
 				while (list($field_name, $field_array) = each($xml_array["fields"])) {
@@ -646,28 +648,28 @@ function graphs() {
 						}else{
 							print "<td " . (($use_javascript == true) ? "onClick='dq_select_line(" . $snmp_query["id"] . ",\"" . encode_data_query_index($snmp_index) . "\");'" : "") . "><span id='text$query_row" . "_" . $column_counter . "'></span></td>";
 						}
-						
+
 						$column_counter++;
 					}
 				}
-				
+
 				print "<td align='right'>";
 				print "<input type='checkbox' name='sg_$query_row' id='sg_$query_row' " . (($use_javascript == true) ? "onClick='dq_update_selection_indicators();'" : "") . ">";
 				print "</td>";
 				print "</tr>\n";
-				
+
 				$row_counter++;
 			}
 			}
 		}else{
 			print "<tr bgcolor='#" . $colors["form_alternate1"] . "'><td colspan='2' style='color: red; font-size: 12px; font-weight: bold;'>Error in data query.</td></tr>\n";
 		}
-		
+
 		print "</table>";
-		
+
 		/* draw the graph template drop down here */
 		$data_query_graphs = db_fetch_assoc("select snmp_query_graph.id,snmp_query_graph.name from snmp_query_graph where snmp_query_graph.snmp_query_id=" . $snmp_query["id"] . " order by snmp_query_graph.name");
-		
+
 		if (sizeof($data_query_graphs) == 1) {
 			form_hidden_box("sgg_" . $snmp_query["id"] . "' id='sgg_" . $snmp_query["id"], $data_query_graphs[0]["id"], "");
 		}elseif (sizeof($data_query_graphs) > 1) {
@@ -679,27 +681,27 @@ function graphs() {
 						<td align='right'>
 							<span style='font-size: 12px; font-style: italic;'>Select a graph type:</span>&nbsp;
 							<select name='sgg_" . $snmp_query["id"] . "' id='sgg_" . $snmp_query["id"] . "' " . (($use_javascript == true) ? "onChange='dq_update_deps(" . $snmp_query["id"] . "," . $num_visible_fields . ");'" : "") . ">
-								"; create_list($data_query_graphs,"name","id","0"); print "
+								"; html_create_list($data_query_graphs,"name","id","0"); print "
 							</select>
 						</td>
 					</tr>
 				</table>";
 		}
-		
+
 		print "<br>";
-		
+
 		if ($use_javascript == true) {
 			print "<script type='text/javascript'>dq_update_deps(" . $snmp_query["id"] . "," . ($num_visible_fields) . ");</script>\n";
 		}
 	}
 	}
-	
+
 	form_hidden_box("save_component_graph", "1", "");
 	form_hidden_box("host_id", $host["id"], "0");
 	form_hidden_box("host_template_id", $host["host_template_id"], "0");
-	
+
 	form_save_button("graphs_new.php");
-	
+
 	print "<script type='text/javascript'>dq_update_selection_indicators();</script>\n";
 	print "<script type='text/javascript'>gt_update_selection_indicators();</script>\n";
 }
