@@ -24,7 +24,7 @@
  +-------------------------------------------------------------------------+
 */
 
-/* make sure these values refect your acual database/host/user/password */
+/* make sure these values refect your actual database/host/user/password */
 $database_type = "mysql";
 $database_default = "cacti";
 $database_hostname = "localhost";
@@ -69,23 +69,53 @@ if ((isset($no_http_headers) ? $no_http_headers : false) != true) {
 	header("Cache-Control: no-store, no-cache, must-revalidate");
 	header("Cache-Control: post-check=0, pre-check=0", false);
 	header("Pragma: no-cache");
-	
+
 	/* initilize php session */
 	session_start();
+
+	/* detect and handle get_magic_quotes */
+	if (!get_magic_quotes_gpc()) {
+		function addslashes_deep($value) {
+			$value = is_array($value) ? array_map('addslashes_deep', $value) : addslashes($value);
+			return $value;
+		}
+
+		$_POST = array_map('addslashes_deep', $_POST);
+		$_GET = array_map('addslashes_deep', $_GET);
+		$_COOKIE = array_map('addslashes_deep', $_COOKIE);
+	}
+
+	/* make sure to start only only Cacti session at a time */
+	if (!isset($_SESSION["cacti_cwd"])) {
+		$_SESSION["cacti_cwd"] = $config["base_path"];
+	}else{
+		if ($_SESSION["cacti_cwd"] != $config["base_path"]) {
+			session_unset();
+			session_destroy();
+		}
+	}
 }
 
 /* display ALL errors */
 error_reporting(E_ALL);
 
-include_once($config["include_path"] . "/config_arrays.php");
-include_once($config["include_path"] . "/config_form.php");
+/* connect to the database server */
+include($config["library_path"] . "/adodb/adodb.inc.php");
+include($config["library_path"] . "/database.php");
+db_connect_real($database_hostname, $database_username, $database_password, $database_default, $database_type);
+
 include_once($config["library_path"] . "/functions.php");
-include_once($config["library_path"] . "/form.php");
-include_once($config["library_path"] . "/database.php");
+include_once($config["include_path"] . "/config_constants.php");
+include_once($config["include_path"] . "/config_arrays.php");
+include_once($config["include_path"] . "/config_settings.php");
+include_once($config["include_path"] . "/config_form.php");
+include_once($config["library_path"] . "/html.php");
+include_once($config["library_path"] . "/html_form.php");
+include_once($config["library_path"] . "/html_utility.php");
 include_once($config["library_path"] . "/variables.php");
 include_once($config["library_path"] . "/auth.php");
 
 /* current cacti version */
-$config["cacti_version"] = "0.8.5a";
+$config["cacti_version"] = "0.8.6b";
 
 ?>
