@@ -24,8 +24,6 @@
  +-------------------------------------------------------------------------+
 */
 
-include($config["include_path"] . "/config_arrays.php");
-
 if (!defined("VALID_HOST_FIELDS")) {
 	define("VALID_HOST_FIELDS", "(hostname|snmp_community|snmp_username|snmp_password|snmp_version|snmp_port|snmp_timeout)");
 }
@@ -304,7 +302,7 @@ $struct_data_source_item = array(
 		"max_length" => "20",
 		"size" => "30",
 		"default" => "600",
-		"description" => "The maximum amount of time that can pass before data is entered as \"unknown\". 
+		"description" => "The maximum amount of time that can pass before data is entered as \"unknown\".
 			(Usually 2x300=600)"
 		),
 	"data_input_field_id" => array(
@@ -375,14 +373,14 @@ $struct_graph = array(
 		"friendly_name" => "Auto Scale",
 		"method" => "checkbox",
 		"default" => "on",
-		"description" => "Auto scale the y-axis instead of defining an upper and lower limit. Note: if this is check both the 
+		"description" => "Auto scale the y-axis instead of defining an upper and lower limit. Note: if this is check both the
 			Upper and Lower limit will be ignored."
 		),
 	"auto_scale_opts" => array(
 		"friendly_name" => "Auto Scale Options",
 		"method" => "radio",
 		"default" => "2",
-		"description" => "Use --alt-autoscale-max to scale to the maximum value, or --alt-autoscale to scale to the absolute 
+		"description" => "Use --alt-autoscale-max to scale to the maximum value, or --alt-autoscale to scale to the absolute
 			minimum and maximum.",
 		"items" => array(
 			0 => array(
@@ -411,15 +409,15 @@ $struct_graph = array(
 		"friendly_name" => "Auto Padding",
 		"method" => "checkbox",
 		"default" => "on",
-		"description" => "Pad text so that legend and graph data always line up. Note: this could cause 
-			graphs to take longer to render because of the larger overhead. Also Auto Padding may not 
+		"description" => "Pad text so that legend and graph data always line up. Note: this could cause
+			graphs to take longer to render because of the larger overhead. Also Auto Padding may not
 			be accurate on all types of graphs, consistant labeling usually helps."
 		),
 	"export" => array(
 		"friendly_name" => "Allow Graph Export",
 		"method" => "checkbox",
 		"default" => "on",
-		"description" => "Choose whether this graph will be included in the static html/png export if you use 
+		"description" => "Choose whether this graph will be included in the static html/png export if you use
 			cacti's export feature."
 		),
 	"upper_limit" => array(
@@ -448,7 +446,7 @@ $struct_graph = array(
 		"method" => "textbox",
 		"max_length" => "50",
 		"default" => "",
-		"description" => "(--unit) Sets the exponent value on the Y-axis for numbers. Note: This option was 
+		"description" => "(--unit) Sets the exponent value on the Y-axis for numbers. Note: This option was
 			recently added in rrdtool 1.0.36."
 		),
 	"unit_exponent_value" => array(
@@ -456,7 +454,7 @@ $struct_graph = array(
 		"method" => "textbox",
 		"max_length" => "50",
 		"default" => "",
-		"description" => "What unit cacti should use on the Y-axis. Use 3 to display everything in 'k' or -6 
+		"description" => "What unit cacti should use on the Y-axis. Use 3 to display everything in 'k' or -6
 			to display everything in 'u' (micro)."
 		),
 	"vertical_label" => array(
@@ -475,10 +473,10 @@ $struct_graph_item = array(
 		"method" => "drop_sql",
 		"sql" => "select
 			CONCAT_WS('',case when host.description is null then 'No Host' when host.description is not null then host.description end,' - ',data_template_data.name,' (',data_template_rrd.data_source_name,')') as name,
-			data_template_rrd.id 
-			from data_template_data,data_template_rrd,data_local 
+			data_template_rrd.id
+			from data_template_data,data_template_rrd,data_local
 			left join host on data_local.host_id=host.id
-			where data_template_rrd.local_data_id=data_local.id 
+			where data_template_rrd.local_data_id=data_local.id
 			and data_template_data.local_data_id=data_local.id
 			order by name",
 		"default" => "0",
@@ -525,7 +523,7 @@ $struct_graph_item = array(
 		"method" => "drop_sql",
 		"sql" => "select id,name from graph_templates_gprint order by name",
 		"default" => "2",
-		"description" => "If this graph item is a GPRINT, you can optionally choose another format 
+		"description" => "If this graph item is a GPRINT, you can optionally choose another format
 			here. You can define additional types under \"GPRINT Presets\"."
 		),
 	"text_format" => array(
@@ -642,29 +640,31 @@ $fields_host_edit = array(
 		),
 	"spacer1" => array(
 		"method" => "spacer",
-		"value" => "SNMP Options"
+		"friendly_name" => "SNMP Options"
 		),
 	"snmp_community" => array(
 		"method" => "textbox",
 		"friendly_name" => "SNMP Community",
 		"description" => "Fill in the SNMP read community for this device.",
 		"value" => "|arg1:snmp_community|",
-		"default" => "public",
 		"form_id" => "|arg1:id|",
+		"default" => read_config_option("snmp_community"),
 		"max_length" => "100",
 		),
 	"snmp_username" => array(
-		"method" => "textbox",
+		"method" => "hidden",
 		"friendly_name" => "SNMP Username",
 		"description" => "Fill in the SNMP username for this device (v3).",
 		"value" => "|arg1:snmp_username|",
+		"default" => read_config_option("snmp_username"),
 		"max_length" => "50",
 		),
 	"snmp_password" => array(
-		"method" => "textbox",
+		"method" => "hidden",
 		"friendly_name" => "SNMP Password",
 		"description" => "Fill in the SNMP password for this device (v3).",
 		"value" => "|arg1:snmp_password|",
+		"default" => read_config_option("snmp_password"),
 		"max_length" => "50",
 		),
 	"snmp_version" => array(
@@ -672,6 +672,7 @@ $fields_host_edit = array(
 		"friendly_name" => "SNMP Version",
 		"description" => "Choose the SNMP version for this host.",
 		"value" => "|arg1:snmp_version|",
+		"default" => read_config_option("snmp_ver"),
 		"array" => $snmp_versions,
 		),
 	"snmp_port" => array(
@@ -679,8 +680,8 @@ $fields_host_edit = array(
 		"friendly_name" => "SNMP Port",
 		"description" => "Enter the UDP port number to use for SNMP (default is 161).",
 		"value" => "|arg1:snmp_port|",
-		"default" => "161",
 		"max_length" => "5",
+		"default" => read_config_option("snmp_port"),
 		"size" => "15"
 		),
 	"snmp_timeout" => array(
@@ -688,8 +689,8 @@ $fields_host_edit = array(
 		"friendly_name" => "SNMP Timeout",
 		"description" => "The maximum number of milliseconds Cacti will wait for an SNMP response (does not work with php-snmp support).",
 		"value" => "|arg1:snmp_timeout|",
-		"default" => "500",
 		"max_length" => "8",
+		"default" => read_config_option("snmp_timeout"),
 		"size" => "15"
 		),
 	"id" => array(
@@ -779,7 +780,7 @@ $fields_rra_edit = array(
 		)
 	);
 
-/* file: snmp.php, action: edit */
+/* file: data_queries.php, action: edit */
 $fields_data_query_edit = array(
 	"name" => array(
 		"method" => "textbox",
@@ -808,7 +809,7 @@ $fields_data_query_edit = array(
 		"friendly_name" => "Data Input Method",
 		"description" => "Choose what type of host, host template this is. The host template will govern what kinds of data should be gathered from this type of host.",
 		"value" => "|arg1:data_input_id|",
-		"sql" => "select id,name from data_input where (type_id=3 or type_id=4) order by name",
+		"sql" => "select id,name from data_input where (type_id=3 or type_id=4 or type_id=5 or type_id=6) order by name",
 		),
 	"id" => array(
 		"method" => "hidden_zero",
@@ -820,7 +821,7 @@ $fields_data_query_edit = array(
 		)
 	);
 
-/* file: snmp.php, action: item_edit */
+/* file: data_queries.php, action: item_edit */
 $fields_data_query_item_edit = array(
 	"name" => array(
 		"method" => "textbox",
@@ -863,6 +864,13 @@ $fields_tree_edit = array(
 		"value" => "|arg1:name|",
 		"max_length" => "255",
 		),
+	"sort_type" => array(
+		"method" => "drop_array",
+		"friendly_name" => "Sorting Type",
+		"description" => "Choose how items in this tree will be sorted.",
+		"value" => "|arg1:sort_type|",
+		"array" => $tree_sort_types,
+		),
 	"id" => array(
 		"method" => "hidden_zero",
 		"value" => "|arg1:id|"
@@ -899,17 +907,18 @@ $fields_user_user_edit_host = array(
 	"grp1" => array(
 		"friendly_name" => "Account Options",
 		"method" => "checkbox_group",
-		"form_id" => "|arg1:id|",
 		"description" => "Set any user account-specific options here.",
 		"items" => array(
 			"must_change_password" => array(
 				"value" => "|arg1:must_change_password|",
 				"friendly_name" => "User Must Change Password at Next Login",
+				"form_id" => "|arg1:id|",
 				"default" => ""
 				),
 			"graph_settings" => array(
 				"value" => "|arg1:graph_settings|",
 				"friendly_name" => "Allow this User to Keep Custom Graph Settings",
+				"form_id" => "|arg1:id|",
 				"default" => "on"
 				)
 			)
@@ -987,4 +996,5 @@ $fields_user_user_edit_host = array(
 		"value" => "1"
 		)
 	);
-?>
+
+?>
\ No newline at end of file
