@@ -25,12 +25,17 @@
 */
 
 include("../include/config.php");
-include("../include/config_settings.php");
 
-$cacti_versions = array("0.8", "0.8.1", "0.8.2", "0.8.2a", "0.8.3", "0.8.3a", "0.8.4", "0.8.5", "0.8.5a");
+/* allow the upgrade script to run for as long as it needs to */
+ini_set("max_execution_time", "0");
+
+$cacti_versions = array("0.8", "0.8.1", "0.8.2", "0.8.2a", "0.8.3", "0.8.3a", "0.8.4", "0.8.5", "0.8.5a", "0.8.6", "0.8.6a", "0.8.6b");
 
 $old_cacti_version = db_fetch_cell("select cacti from version");
 
+/* try to find current (old) version in the array */
+$old_version_index = array_search($old_cacti_version, $cacti_versions);
+
 /* do a version check */
 if ($old_cacti_version == $config["cacti_version"]) {
 	print "	<p style='font-family: Verdana, Arial; font-size: 16px; font-weight: bold; color: red;'>Error</p>
@@ -52,19 +57,19 @@ if ($old_cacti_version == $config["cacti_version"]) {
 
 function db_install_execute($cacti_version, $sql) {
 	$sql_install_cache = (isset($_SESSION["sess_sql_install_cache"]) ? $_SESSION["sess_sql_install_cache"] : array());
-	
+
 	if (db_execute($sql)) {
 		$sql_install_cache{sizeof($sql_install_cache)}[$cacti_version][1] = $sql;
 	}else{
 		$sql_install_cache{sizeof($sql_install_cache)}[$cacti_version][0] = $sql;
 	}
-	
+
 	$_SESSION["sess_sql_install_cache"] = $sql_install_cache;
 }
 
 function find_best_path($binary_name) {
 	$search_paths = array("/bin", "/sbin", "/usr/bin", "/usr/sbin", "/usr/local/bin", "/usr/local/sbin");
-	
+
 	for ($i=0; $i<count($search_paths); $i++) {
 		if ((file_exists($search_paths[$i] . "/" . $binary_name)) && (is_executable($search_paths[$i] . "/" . $binary_name))) {
 			return $search_paths[$i] . "/" . $binary_name;
@@ -72,16 +77,6 @@ function find_best_path($binary_name) {
 	}
 }
 
-$current_document_root = "";
-$current_document = str_replace("/install", "", dirname($_SERVER["PHP_SELF"]));
-
-/* find the current document root depending on if we're using apache or iis */
-if ((stristr($_SERVER["SERVER_SOFTWARE"], "IIS")) && (isset($_SERVER["PATH_TRANSLATED"]))) {
-	$current_document_root = str_replace($_SERVER["PHP_SELF"], "", str_replace("\\", "/", $_SERVER["PATH_TRANSLATED"]));
-}elseif (isset($_SERVER["DOCUMENT_ROOT"])) {
-	$current_document_root = $_SERVER["DOCUMENT_ROOT"];
-}
-
 /* Here, we define each name, default value, type, and path check for each value
 we want the user to input. The "name" field must exist in the 'settings' table for
 this to work. Cacti also uses different default values depending on what OS it is
@@ -92,7 +87,7 @@ $input["path_rrdtool"] = $settings["path"]["path_rrdtool"];
 
 if ($config["cacti_server_os"] == "unix") {
 	$which_rrdtool = find_best_path("rrdtool");
-	
+
 	if (!empty($which_rrdtool)) {
 		$input["path_rrdtool"]["default"] = $which_rrdtool;
 	}elseif (read_config_option("path_rrdtool") != "<DEFAULT>") {
@@ -109,7 +104,7 @@ $input["path_php_binary"] = $settings["path"]["path_php_binary"];
 
 if ($config["cacti_server_os"] == "unix") {
 	$which_php = find_best_path("php");
-	
+
 	if (!empty($which_php)) {
 		$input["path_php_binary"]["default"] = $which_php;
 	}elseif (read_config_option("path_php_binary") != "<DEFAULT>") {
@@ -124,9 +119,9 @@ if ($config["cacti_server_os"] == "unix") {
 /* snmpwalk Binary Path */
 if ($config["cacti_server_os"] == "unix") {
 	$input["path_snmpwalk"] = $settings["path"]["path_snmpwalk"];
-	
+
 	$which_snmpwalk = find_best_path("snmpwalk");
-	
+
 	if (!empty($which_snmpwalk)) {
 		$input["path_snmpwalk"]["default"] = $which_snmpwalk;
 	}elseif (read_config_option("path_snmpwalk") != "<DEFAULT>") {
@@ -139,9 +134,9 @@ if ($config["cacti_server_os"] == "unix") {
 /* snmpget Binary Path */
 if ($config["cacti_server_os"] == "unix") {
 	$input["path_snmpget"] = $settings["path"]["path_snmpget"];
-	
+
 	$which_snmpget = find_best_path("snmpget");
-	
+
 	if (!empty($which_snmpget)) {
 		$input["path_snmpget"]["default"] = $which_snmpget;
 	}elseif (read_config_option("path_snmpget") != "<DEFAULT>") {
@@ -151,6 +146,11 @@ if ($config["cacti_server_os"] == "unix") {
 	}
 }
 
+/* log file path */
+$input["path_cactilog"] = $settings["path"]["path_cactilog"];
+$input["path_cactilog"]["description"] = "The path to your Cacti log file.";
+$input["path_cactilog"]["default"] = $config["base_path"] . "/log/cacti.log";
+
 /* SNMP Version */
 if ($config["cacti_server_os"] == "unix") {
 	$input["snmp_version"] = $settings["general"]["snmp_version"];
@@ -179,8 +179,12 @@ if (empty($_REQUEST["step"])) {
 		$_REQUEST["step"] = "3";
 	}elseif (($_REQUEST["step"] == "2") && ($_REQUEST["install_type"] == "3")) {
 		$_REQUEST["step"] = "8";
+	}elseif (($_REQUEST["step"] == "8") && ($old_version_index <= array_search("0.8.5a", $cacti_versions))) {
+		$_REQUEST["step"] = "9";
 	}elseif ($_REQUEST["step"] == "8") {
 		$_REQUEST["step"] = "3";
+	}elseif ($_REQUEST["step"] == "9") {
+		$_REQUEST["step"] = "3";
 	}elseif ($_REQUEST["step"] == "3") {
 		$_REQUEST["step"] = "4";
 	}
@@ -189,48 +193,45 @@ if (empty($_REQUEST["step"])) {
 if ($_REQUEST["step"] == "4") {
 	include_once("../lib/data_query.php");
 	include_once("../lib/utility.php");
-	
+
 	$i = 0;
-	
+
 	/* get all items on the form and write values for them  */
 	while (list($name, $array) = each($input)) {
 		if (isset($_POST[$name])) {
 			db_execute("update settings set value='" . $_POST[$name] . "' where name='$name'");
 		}
 	}
-	
+
 	setcookie(session_name(),"",time() - 3600,"/");
-	
+
 	kill_session_var("sess_config_array");
 	kill_session_var("sess_host_cache_array");
-	
+
 	/* just in case we have hard drive graphs to deal with */
-	data_query(db_fetch_cell("select id from host where hostname='127.0.0.1'"), 6);
-	
+	run_data_query(db_fetch_cell("select id from host where hostname='127.0.0.1'"), 6);
+
 	/* it's always a good idea to re-populate the poller cache to make sure everything is refreshed and
 	up-to-date */
 	repopulate_poller_cache();
-	
+
 	db_execute("delete from version");
 	db_execute("insert into version (cacti) values ('" . $config["cacti_version"] . "')");
-	
+
 	header ("Location: ../index.php");
 	exit;
 }elseif (($_REQUEST["step"] == "8") && ($_REQUEST["install_type"] == "3")) {
-	/* try to find current (old) version in the array */
-	$version_index = array_search($old_cacti_version, $cacti_versions);
-	
 	/* if the version is not found, die */
-	if (!is_int(array_search($old_cacti_version, $cacti_versions))) {
+	if (!is_int($old_version_index)) {
 		print "	<p style='font-family: Verdana, Arial; font-size: 16px; font-weight: bold; color: red;'>Error</p>
-			<p style='font-family: Verdana, Arial; font-size: 12px;'>Invalid Cacti version 
+			<p style='font-family: Verdana, Arial; font-size: 12px;'>Invalid Cacti version
 			<strong>$old_cacti_version</strong>, cannot upgrade to <strong>" . $config["cacti_version"] . "
 			</strong></p>";
 		exit;
 	}
-	
+
 	/* loop from the old version to the current, performing updates for each version in between */
-	for ($i=($version_index+1); $i<count($cacti_versions); $i++) {
+	for ($i=($old_version_index+1); $i<count($cacti_versions); $i++) {
 		if ($cacti_versions[$i] == "0.8.1") {
 			include ("0_8_to_0_8_1.php");
 			upgrade_to_0_8_1();
@@ -251,8 +252,21 @@ if ($_REQUEST["step"] == "4") {
 			include ("0_8_4_to_0_8_5.php");
 			upgrade_to_0_8_5();
 		}elseif ($cacti_versions[$i] == "0.8.5a") {
-			if ($old_cacti_version == "0.8.5") {
-				/* no database upgrades for 0.8.5 -> 0.8.5a */
+			/* no database upgrades for 0.8.5 -> 0.8.5a */
+		}elseif ($cacti_versions[$i] == "0.8.6") {
+			include ("0_8_5a_to_0_8_6.php");
+			upgrade_to_0_8_6();
+		}elseif ($cacti_versions[$i] == "0.8.6a") {
+			include ("0_8_6_to_0_8_6a.php");
+			upgrade_to_0_8_6a();
+
+			if ($old_cacti_version == "0.8.6") {
+				/* no database upgrades for 0.8.6 -> 0.8.6a */
+				$_REQUEST["step"] = "3";
+			}
+		}elseif ($cacti_versions[$i] == "0.8.6b") {
+			if ($old_cacti_version == "0.8.6a") {
+				/* no database upgrades for 0.8.6a -> 0.8.6b */
 				$_REQUEST["step"] = "3";
 			}
 		}
@@ -270,12 +284,12 @@ if ($_REQUEST["step"] == "4") {
 			font-size: 10pt;
 			font-family: Verdana, Arial, sans-serif;
 		}
-		
+
 		.code
 		{
 			font-family: Courier New, Courier;
 		}
-		
+
 		.header-text
 		{
 			color: white;
@@ -302,53 +316,53 @@ if ($_REQUEST["step"] == "4") {
 				<tr>
 					<td width="100%" style="font-size: 12px;">
 						<?php if ($_REQUEST["step"] == "1") { ?>
-						
-						<p>Thanks for taking the time to download and install cacti, the complete graphing 
-						solution for your network. Before you can start making cool graphs, there are a few 
+
+						<p>Thanks for taking the time to download and install cacti, the complete graphing
+						solution for your network. Before you can start making cool graphs, there are a few
 						pieces of data that cacti needs to know.</p>
-						
+
 						<p>Make sure you have read and followed the required steps needed to install cacti
-						before continuing. Install information can be found for 
+						before continuing. Install information can be found for
 						<a href="../docs/html/install_unix.html">Unix</a> and <a href="../docs/html/install_windows.html">Win32</a>-based operating systems.</p>
-						
+
 						<p>Also, if this is an upgrade, be sure to reading the <a href="../docs/html/upgrade.html">Upgrade</a> information file.</p>
-						
+
 						<p>Cacti is licensed under the GNU General Public License, you must agree
 						to its provisions before continuing:</p>
-						
+
 						<p class="code">This program is free software; you can redistribute it and/or
 						modify it under the terms of the GNU General Public License
 						as published by the Free Software Foundation; either version 2
 						of the License, or (at your option) any later version.</p>
-						
+
 						<p class="code">This program is distributed in the hope that it will be useful,
 						but WITHOUT ANY WARRANTY; without even the implied warranty of
 						MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 						GNU General Public License for more details.</p>
-						
+
 						<?php }elseif ($_REQUEST["step"] == "2") { ?>
-						
+
 						<p>Please select the type of installation</p>
-						
+
 						<p>
 						<select name="install_type">
 							<option value="1"<?php print ($default_install_type == "1") ? " selected" : "";?>>New Install</option>
 							<option value="3"<?php print ($default_install_type == "3") ? " selected" : "";?>>Upgrade from cacti 0.8.x</option>
 						</select>
 						</p>
-						
-						<p>The following information has been determined from cacti's configuration file.
+
+						<p>The following information has been determined from Cacti's configuration file.
 						If it is not correct, please edit 'include/config.php' before continuing.</p>
-						
+
 						<p class="code">
 						<?php	print "Database User: $database_username<br>";
 							print "Database Hostname: $database_hostname<br>";
 							print "Database: $database_default<br>";
 							print "Server Operating System Type: " . $config["cacti_server_os"] . "<br>"; ?>
 						</p>
-						
+
 						<?php }elseif ($_REQUEST["step"] == "3") { ?>
-						
+
 						<p>Make sure all of these values are correct before continuing.</p>
 						<?php
 						$i = 0;
@@ -357,11 +371,11 @@ if ($_REQUEST["step"] == "4") {
 						while (list($name, $array) = each($input)) {
 							if (isset($input[$name])) {
 								$current_value = $array["default"];
-								
+
 								/* run a check on the path specified only if specified above, then fill a string with
 								the results ('FOUND' or 'NOT FOUND') so they can be displayed on the form */
 								$form_check_string = "";
-								
+
 								if ($array["method"] == "textbox") {
 									if (@file_exists($current_value)) {
 										$form_check_string = "<font color='#008000'>[FOUND]</font> ";
@@ -369,18 +383,18 @@ if ($_REQUEST["step"] == "4") {
 										$form_check_string = "<font color='#FF0000'>[NOT FOUND]</font> ";
 									}
 								}
-								
+
 								/* draw the acual header and textbox on the form */
 								print "<p><strong>" . $form_check_string . $array["friendly_name"] . "</strong>";
-								
+
 								if (!empty($array["friendly_name"])) {
 									print ": " . $array["description"];
 								}else{
 									print "<strong>" . $array["description"] . "</strong>";
 								}
-								
+
 								print "<br>";
-								
+
 								switch ($array["method"]) {
 								case 'textbox':
 									form_text_box($name, $current_value, "", "", "40", "text");
@@ -389,52 +403,82 @@ if ($_REQUEST["step"] == "4") {
 									form_dropdown($name, $array["array"], "", "", $current_value, "", "");
 									break;
 								}
-								
+
 								print "<br></p>";
 							}
-							
+
 							$i++;
 						}?>
-						
+
 						<p><strong><font color="#FF0000">NOTE:</font></strong> Once you click "Finish",
 						all of your settings will be saved and your database will be upgraded if this
 						is an upgrade. You can change any of the settings on this screen at a later
 						time by going to "Cacti Settings" from within Cacti.</p>
-						
+
 						<?php }elseif ($_REQUEST["step"] == "8") { ?>
-						
+
 						<p>Upgrade results:</p>
-						
+
 						<?php
 						$current_version  = "";
-						
+						$upgrade_results = "";
+						$failed_sql_query = false;
+
 						$fail_text = "<span style='color: red; font-weight: bold; font-size: 12px;'>[Fail]</span>&nbsp;";
 						$success_text = "<span style='color: green; font-weight: bold; font-size: 12px;'>[Success]</span>&nbsp;";
-						
+
 						if (isset($_SESSION["sess_sql_install_cache"])) {
 							while (list($index, $arr1) = each($_SESSION["sess_sql_install_cache"])) {
 								while (list($version, $arr2) = each($arr1)) {
 									while (list($status, $sql) = each($arr2)) {
 										if ($current_version != $version) {
 											$version_index = array_search($version, $cacti_versions);
-											print "<p><strong>" . $cacti_versions{$version_index-1}  . " -> " . $cacti_versions{$version_index} . "</strong></p>\n";
+											$upgrade_results .= "<p><strong>" . $cacti_versions{$version_index-1}  . " -> " . $cacti_versions{$version_index} . "</strong></p>\n";
 										}
-										
-										print "<p class='code'>" . (($status == 0) ? $fail_text : $success_text) . nl2br($sql) . "</p>\n";
-										
+
+										$upgrade_results .= "<p class='code'>" . (($status == 0) ? $fail_text : $success_text) . nl2br($sql) . "</p>\n";
+
+										/* if there are one or more failures, make a note because we are going to print
+										out a warning to the user later on */
+										if ($status == 0) {
+											$failed_sql_query = true;
+										}
+
 										$current_version = $version;
 									}
 								}
 							}
-						
+
 							kill_session_var("sess_sql_install_cache");
 						}else{
 							print "<em>No SQL queries have been executed.</em>";
 						}
+
+						if ($failed_sql_query == true) {
+							print "<p><strong><font color='#FF0000'>WARNING:</font></strong> One or more of the SQL queries needed to
+								upgraded your Cacti installation has failed. Please see below for more details. Your
+								Cacti MySQL user must have <strong>SELECT, INSERT, UPDATE, DELETE, ALTER, CREATE, and DROP</strong>
+								permissions. You should try executing the failed queries as 'root' to ensure that you do not have
+								a permissions problem.</p>\n";
+						}
+
+						print $upgrade_results;
 						?>
-						
+
+						<?php }elseif ($_REQUEST["step"] == "9") { ?>
+
+						<p style='font-size: 16px; font-weight: bold; color: red;'>Important Upgrade Notice</p>
+
+						<p>Before you continue with the installation, you <strong>must</strong> update your <tt>/etc/crontab</tt> file to point to <tt>poller.php</tt> instead of <tt>cmd.php</tt>.</p>
+
+						<p>See the sample crontab entry below with the change made in red. Your crontab line will look slightly different based upon your setup.</p>
+
+						<p><tt>*/5 * * * * cactiuser php /var/www/html/cacti/<span style='font-weight: bold; color: red;'>poller.php</span> &gt; /dev/null 2&gt;&amp;1</tt></p>
+
+						<p>Once you have made this change, please click Next to continue.</p>
+
 						<?php }?>
-						
+
 						<p align="right"><input type="image" src="install_<?php if ($_REQUEST["step"] == "3") {?>finish<?php }else{?>next<?php }?>.gif" alt="<?php if ($_REQUEST["step"] == "3"){?>Finish<?php }else{?>Next<?php }?>"></p>
 					</td>
 				</tr>
