@@ -1,16 +1,19 @@
 <?php
 /* 
-V4.05 13 Dec 2003  (c) 2000-2003 John Lim (jlim@natsoft.com.my). All rights reserved.
+V4.23 16 June 2004  (c) 2000-2004 John Lim (jlim@natsoft.com.my). All rights reserved.
 Released under both BSD license and Lesser GPL library license. 
 Whenever there is any discrepancy between the two licenses, 
 the BSD license will take precedence. See License.txt. 
 Set tabs to 4 for best viewing.
   
-  Latest version is available at http://php.weblogs.com/
+  Latest version is available at http://adodb.sourceforge.net
   
 	Microsoft Access ADO data driver. Requires ADO and ODBC. Works only on MS Windows.
 */
 
+// security - hide paths
+if (!defined('ADODB_DIR')) die();
+
 if (!defined('_ADODB_ADO_LAYER')) {
 	include(ADODB_DIR."/drivers/adodb-ado.inc.php");
 }
