@@ -1,6 +1,6 @@
 <?php
 /**
-* @version V4.05 13 Dec 2003 (c) 2000-2003 John Lim (jlim@natsoft.com.my). All rights reserved.
+* @version V4.23 16 June 2004 (c) 2000-2004 John Lim (jlim@natsoft.com.my). All rights reserved.
 * Released under both BSD license and Lesser GPL library license.
 * Whenever there is any discrepancy between the two licenses,
 * the BSD license will take precedence.
@@ -12,6 +12,10 @@
 * Informix 9 driver that supports SELECT FIRST
 *
 */
+
+// security - hide paths
+if (!defined('ADODB_DIR')) die();
+
 include_once(ADODB_DIR.'/drivers/adodb-informix72.inc.php');
 
 class ADODB_informix extends ADODB_informix72 {
