@@ -1,6 +1,6 @@
 <?php
 /*
- V4.05 13 Dec 2003  (c) 2000-2003 John Lim (jlim@natsoft.com.my). All rights reserved.
+ V4.23 16 June 2004  (c) 2000-2004 John Lim (jlim@natsoft.com.my). All rights reserved.
   Released under both BSD license and Lesser GPL library license. 
   Whenever there is any discrepancy between the two licenses, 
   the BSD license will take precedence.
@@ -42,6 +42,9 @@
 
 */
 
+// security - hide paths
+if (!defined('ADODB_DIR')) die();
+
 function adodb_addslashes($s)
 {
 	$len = strlen($s);
@@ -58,7 +61,10 @@ class ADODB_postgres64 extends ADOConnection{
 	var $_resultid = false;
   	var $concat_operator='||';
 	var $metaDatabasesSQL = "select datname from pg_database where datname not in ('template0','template1') order by 1";
-    var $metaTablesSQL = "select tablename,'T' from pg_tables where tablename not like 'pg\_%' union 
+    var $metaTablesSQL = "select tablename,'T' from pg_tables where tablename not like 'pg\_%'
+	and tablename not in ('sql_features', 'sql_implementation_info', 'sql_languages',
+	 'sql_packages', 'sql_sizing', 'sql_sizing_profiles') 
+	union 
         select viewname,'V' from pg_views where viewname not like 'pg\_%'";
 	//"select tablename from pg_tables where tablename not like 'pg_%' order by 1";
 	var $isoDates = true; // accepts dates in ISO format
@@ -69,8 +75,17 @@ class ADODB_postgres64 extends ADOConnection{
 		FROM pg_class c, pg_attribute a,pg_type t 
 		WHERE relkind = 'r' AND (c.relname='%s' or c.relname = lower('%s')) and a.attname not like '....%%'
 AND a.attnum > 0 AND a.atttypid = t.oid AND a.attrelid = c.oid ORDER BY a.attnum";
+
+	var $metaColumnsSQL1 = "SELECT a.attname, t.typname, a.attlen, a.atttypmod, a.attnotnull, a.atthasdef, a.attnum 
+FROM pg_class c, pg_attribute a, pg_type t, pg_namespace n 
+WHERE relkind = 'r' AND (c.relname='%s' or c.relname = lower('%s'))
+ and c.relnamespace=n.oid and n.nspname='%s' 
+	and a.attname not like '....%%' AND a.attnum > 0 
+	AND a.atttypid = t.oid AND a.attrelid = c.oid ORDER BY a.attnum";
+	
 	// get primary key etc -- from Freek Dijkstra
-	var $metaKeySQL = "SELECT ic.relname AS index_name, a.attname AS column_name,i.indisunique AS unique_key, i.indisprimary AS primary_key FROM pg_class bc, pg_class ic, pg_index i, pg_attribute a WHERE bc.oid = i.indrelid AND ic.oid = i.indexrelid AND (i.indkey[0] = a.attnum OR i.indkey[1] = a.attnum OR i.indkey[2] = a.attnum OR i.indkey[3] = a.attnum OR i.indkey[4] = a.attnum OR i.indkey[5] = a.attnum OR i.indkey[6] = a.attnum OR i.indkey[7] = a.attnum) AND a.attrelid = bc.oid AND bc.relname = '%s'";
+	var $metaKeySQL = "SELECT ic.relname AS index_name, a.attname AS column_name,i.indisunique AS unique_key, i.indisprimary AS primary_key 
+	FROM pg_class bc, pg_class ic, pg_index i, pg_attribute a WHERE bc.oid = i.indrelid AND ic.oid = i.indexrelid AND (i.indkey[0] = a.attnum OR i.indkey[1] = a.attnum OR i.indkey[2] = a.attnum OR i.indkey[3] = a.attnum OR i.indkey[4] = a.attnum OR i.indkey[5] = a.attnum OR i.indkey[6] = a.attnum OR i.indkey[7] = a.attnum) AND a.attrelid = bc.oid AND bc.relname = '%s'";
 	
 	var $hasAffectedRows = true;
 	var $hasLimit = false;	// set to true for pgsql 7 only. support pgsql/mysql SELECT * FROM TABLE LIMIT 10
@@ -88,6 +103,8 @@ AND a.attnum > 0 AND a.atttypid = t.oid AND a.attrelid = c.oid ORDER BY a.attnum
 	var $random = 'random()';		/// random function
 	var $autoRollback = true; // apparently pgsql does not autorollback properly before 4.3.4
 							// http://bugs.php.net/bug.php?id=25404
+							
+	var $_bindInputArray = false; // requires postgresql 7.3+ and ability to modify database
 	
 	// The last (fmtTimeStamp is not entirely correct: 
 	// PostgreSQL also has support for time zones, 
@@ -349,6 +366,8 @@ select viewname,'V' from pg_views where viewname like $mask";
 	function BlobEncode($blob)
 	{
 		if (ADODB_PHPVER >= 0x4200) return pg_escape_bytea($blob);
+		
+		/*92=backslash, 0=null, 39=single-quote*/
 		$badch = array(chr(92),chr(0),chr(39)); # \  null  '
 		$fixch = array('\\\\134','\\\\000','\\\\047');
 		return adodb_str_replace($badch,$fixch,$blob);
@@ -358,8 +377,8 @@ select viewname,'V' from pg_views where viewname like $mask";
 	
 	function UpdateBlob($table,$column,$val,$where,$blobtype='BLOB')
 	{
-		return $this->Execute("UPDATE $table SET $column=? WHERE $where",
-			array($this->BlobEncode($val))) != false;
+		// do not use bind params which uses qstr(), as blobencode() already quotes data
+		return $this->Execute("UPDATE $table SET $column='".$this->BlobEncode($val)."'::bytea WHERE $where");
 	}
 	
 	function OffsetDate($dayFraction,$date=false)
@@ -369,123 +388,139 @@ select viewname,'V' from pg_views where viewname like $mask";
 	}
 	
 
-	// converts field names to lowercase 
-	function &MetaColumns($table) 
+	// for schema support, pass in the $table param "$schema.$tabname".
+	// converts field names to lowercase, $upper is ignored
+	function &MetaColumns($table,$upper=true) 
 	{
 	global $ADODB_FETCH_MODE;
 	
-		//if (strncmp(PHP_OS,'WIN',3) === 0);
+		$schema = false;
+		$this->_findschema($table,$schema);
+		
 		$table = strtolower($table);
-	
-		if (!empty($this->metaColumnsSQL)) { 
-			$save = $ADODB_FETCH_MODE;
-			$ADODB_FETCH_MODE = ADODB_FETCH_NUM;
-			if ($this->fetchMode !== false) $savem = $this->SetFetchMode(false);
-			$rs = $this->Execute(sprintf($this->metaColumnsSQL,$table,$table));
+
+		$save = $ADODB_FETCH_MODE;
+		$ADODB_FETCH_MODE = ADODB_FETCH_NUM;
+		if ($this->fetchMode !== false) $savem = $this->SetFetchMode(false);
+		
+		if ($schema) $rs =& $this->Execute(sprintf($this->metaColumnsSQL1,$table,$table,$schema));
+		else $rs =& $this->Execute(sprintf($this->metaColumnsSQL,$table,$table));
+		if (isset($savem)) $this->SetFetchMode($savem);
+		$ADODB_FETCH_MODE = $save;
+		
+		if ($rs === false) return false;
+		
+		if (!empty($this->metaKeySQL)) {
+			// If we want the primary keys, we have to issue a separate query
+			// Of course, a modified version of the metaColumnsSQL query using a 
+			// LEFT JOIN would have been much more elegant, but postgres does 
+			// not support OUTER JOINS. So here is the clumsy way.
+			
+			$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
+			
+			$rskey = $this->Execute(sprintf($this->metaKeySQL,($table)));
+			// fetch all result in once for performance.
+			$keys =& $rskey->GetArray();
 			if (isset($savem)) $this->SetFetchMode($savem);
 			$ADODB_FETCH_MODE = $save;
 			
-			if ($rs === false) return false;
-			
-			if (!empty($this->metaKeySQL)) {
-				// If we want the primary keys, we have to issue a separate query
-				// Of course, a modified version of the metaColumnsSQL query using a 
-				// LEFT JOIN would have been much more elegant, but postgres does 
-				// not support OUTER JOINS. So here is the clumsy way.
-				
-				$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
-				
-				$rskey = $this->Execute(sprintf($this->metaKeySQL,($table)));
-				// fetch all result in once for performance.
-				$keys =& $rskey->GetArray();
-				if (isset($savem)) $this->SetFetchMode($savem);
-				$ADODB_FETCH_MODE = $save;
-				
-				$rskey->Close();
-				unset($rskey);
-			}
-
-			$rsdefa = array();
-			if (!empty($this->metaDefaultsSQL)) {
-				$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
-				$sql = sprintf($this->metaDefaultsSQL, ($table));
-				$rsdef = $this->Execute($sql);
-				if (isset($savem)) $this->SetFetchMode($savem);
-				$ADODB_FETCH_MODE = $save;
-				
-				if ($rsdef) {
-					while (!$rsdef->EOF) {
-						$num = $rsdef->fields['num'];
-						$s = $rsdef->fields['def'];
-						if (substr($s, 0, 1) == "'") { /* quoted strings hack... for now... fixme */
-							$s = substr($s, 1);
-							$s = substr($s, 0, strlen($s) - 1);
-						}
-	
-						$rsdefa[$num] = $s;
-						$rsdef->MoveNext();
-					}
-				} else {
-					ADOConnection::outp( "==> SQL => " . $sql);
-				}
-				unset($rsdef);
-			}
-		
-			$retarr = array();
-			while (!$rs->EOF) { 	
-				$fld = new ADOFieldObject();
-				$fld->name = $rs->fields[0];
-				$fld->type = $rs->fields[1];
-				$fld->max_length = $rs->fields[2];
-				if ($fld->max_length <= 0) $fld->max_length = $rs->fields[3]-4;
-				if ($fld->max_length <= 0) $fld->max_length = -1;
-				
-				// dannym
-				// 5 hasdefault; 6 num-of-column
-				$fld->has_default = ($rs->fields[5] == 't');
-				if ($fld->has_default) {
-					$fld->default_value = $rsdefa[$rs->fields[6]];
-				}
-
-				//Freek
-				if ($rs->fields[4] == $this->true) {
-					$fld->not_null = true;
-				}
-				
-				// Freek
-				if (is_array($keys)) {
-					foreach($keys as $key) {
-						if ($fld->name == $key['column_name'] AND $key['primary_key'] == $this->true) 
-							$fld->primary_key = true;
-						if ($fld->name == $key['column_name'] AND $key['unique_key'] == $this->true) 
-							$fld->unique = true; // What name is more compatible?
-					}
-				}
-				
-				if ($ADODB_FETCH_MODE == ADODB_FETCH_NUM) $retarr[] = $fld;	
-				else $retarr[strtoupper($fld->name)] = $fld;
-				
-				$rs->MoveNext();
-			}
-			$rs->Close();
-			return $retarr;	
+			$rskey->Close();
+			unset($rskey);
 		}
-		return false;
+
+		$rsdefa = array();
+		if (!empty($this->metaDefaultsSQL)) {
+			$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
+			$sql = sprintf($this->metaDefaultsSQL, ($table));
+			$rsdef = $this->Execute($sql);
+			if (isset($savem)) $this->SetFetchMode($savem);
+			$ADODB_FETCH_MODE = $save;
+			
+			if ($rsdef) {
+				while (!$rsdef->EOF) {
+					$num = $rsdef->fields['num'];
+					$s = $rsdef->fields['def'];
+					if (strpos($s,'::')===false && substr($s, 0, 1) == "'") { /* quoted strings hack... for now... fixme */
+						$s = substr($s, 1);
+						$s = substr($s, 0, strlen($s) - 1);
+					}
+
+					$rsdefa[$num] = $s;
+					$rsdef->MoveNext();
+				}
+			} else {
+				ADOConnection::outp( "==> SQL => " . $sql);
+			}
+			unset($rsdef);
+		}
+	
+		$retarr = array();
+		while (!$rs->EOF) { 	
+			$fld = new ADOFieldObject();
+			$fld->name = $rs->fields[0];
+			$fld->type = $rs->fields[1];
+			$fld->max_length = $rs->fields[2];
+			if ($fld->max_length <= 0) $fld->max_length = $rs->fields[3]-4;
+			if ($fld->max_length <= 0) $fld->max_length = -1;
+			
+			// dannym
+			// 5 hasdefault; 6 num-of-column
+			$fld->has_default = ($rs->fields[5] == 't');
+			if ($fld->has_default) {
+				$fld->default_value = $rsdefa[$rs->fields[6]];
+			}
+
+			//Freek
+			if ($rs->fields[4] == $this->true) {
+				$fld->not_null = true;
+			}
+			
+			// Freek
+			if (is_array($keys)) {
+				foreach($keys as $key) {
+					if ($fld->name == $key['column_name'] AND $key['primary_key'] == $this->true) 
+						$fld->primary_key = true;
+					if ($fld->name == $key['column_name'] AND $key['unique_key'] == $this->true) 
+						$fld->unique = true; // What name is more compatible?
+				}
+			}
+			
+			if ($ADODB_FETCH_MODE == ADODB_FETCH_NUM) $retarr[] = $fld;	
+			else $retarr[($upper) ? strtoupper($fld->name) : $fld->name] = $fld;
+			
+			$rs->MoveNext();
+		}
+		$rs->Close();
+		return $retarr;	
+		
 	}
 
 	  function &MetaIndexes ($table, $primary = FALSE)
-        {
-                global $ADODB_FETCH_MODE;
+      {
+         global $ADODB_FETCH_MODE;
                 
-                $sql = '
+				$schema = false;
+				$this->_findschema($table,$schema);
+
+				if ($schema) { // requires pgsql 7.3+ - pg_namespace used.
+					$sql = '
+SELECT c.relname as "Name", i.indisunique as "Unique", i.indkey as "Columns" 
+FROM pg_catalog.pg_class c 
+JOIN pg_catalog.pg_index i ON i.indexrelid=c.oid 
+JOIN pg_catalog.pg_class c2 ON c2.oid=i.indrelid
+	,pg_namespace n 
+WHERE (c2.relname=\'%s\' or c2.relname=lower(\'%s\')) and c.relnamespace=c2.relnamespace and c.relnamespace=n.oid and n.nspname=\'%s\' AND i.indisprimary=false';
+				} else {
+	                $sql = '
 SELECT c.relname as "Name", i.indisunique as "Unique", i.indkey as "Columns"
 FROM pg_catalog.pg_class c
 JOIN pg_catalog.pg_index i ON i.indexrelid=c.oid
 JOIN pg_catalog.pg_class c2 ON c2.oid=i.indrelid
-WHERE c2.relname=\'%s\'';
-                
+WHERE c2.relname=\'%s\' or c2.relname=lower(\'%s\')';
+    			}
+				            
                 if ($primary == FALSE) {
-                        $sql .= ' AND i.indisprimary=false;';
+                	$sql .= ' AND i.indisprimary=false;';
                 }
                 
                 $save = $ADODB_FETCH_MODE;
@@ -494,23 +529,20 @@ WHERE c2.relname=\'%s\'';
                         $savem = $this->SetFetchMode(FALSE);
                 }
                 
-                $rs = $this->Execute(sprintf($sql,$table));
-                
+                $rs = $this->Execute(sprintf($sql,$table,$table,$schema));
                 if (isset($savem)) {
                         $this->SetFetchMode($savem);
                 }
                 $ADODB_FETCH_MODE = $save;
-                
+
                 if (!is_object($rs)) {
-                        return FALSE;
+                	return FALSE;
                 }
-                
-                $col_names = $this->MetaColumnNames($table);
+				
+                $col_names = $this->MetaColumnNames($table,true);
                 $indexes = array();
-                
                 while ($row = $rs->FetchRow()) {
                         $columns = array();
-                        
                         foreach (explode(' ', $row[2]) as $col) {
                                 $columns[] = $col_names[$col - 1];
                         }
@@ -520,7 +552,6 @@ WHERE c2.relname=\'%s\'';
                                 'columns' => $columns
                         );
                 }
-                
                 return $indexes;
         }
 
@@ -532,7 +563,7 @@ WHERE c2.relname=\'%s\'';
 	function _connect($str,$user='',$pwd='',$db='',$ctype=0)
 	{
 		
-		if (!function_exists('pg_pconnect')) return false;
+		if (!function_exists('pg_pconnect')) return null;
 		
 		$this->_errorMsg = false;
 		
@@ -586,16 +617,44 @@ WHERE c2.relname=\'%s\'';
 	{
 		return $this->_connect($str,$user,$pwd,$db,1);
 	}
+	
 
 	// returns queryID or false
 	function _query($sql,$inputarr)
 	{
+		
+		if ($inputarr) {
 		/*
-		if (is_array($sql)) {
-			if (!$sql[1]) {
+			It appears that PREPARE/EXECUTE is slower for many queries.
 			
-				$sqltxt = $sql[0];
-				$plan = $sql[1] = 'P'.md5($sqltxt);
+			For query executed 1000 times:
+			"select id,firstname,lastname from adoxyz 
+				where firstname not like ? and lastname not like ? and id = ?"
+				
+			with plan = 1.51861286163 secs
+			no plan =   1.26903700829 secs
+
+			
+
+		*/
+			$plan = 'P'.md5($sql);
+				
+			$execp = '';
+			foreach($inputarr as $v) {
+				if ($execp) $execp .= ',';
+				if (is_string($v)) {
+					if (strncmp($v,"'",1) !== 0) $execp .= $this->qstr($v);
+				} else {
+					$execp .= $v;
+				}
+			}
+			
+			if ($execp) $exsql = "EXECUTE $plan ($execp)";
+			else $exsql = "EXECUTE $plan";
+			
+			$rez = @pg_exec($this->_connectionID,$exsql);
+			if (!$rez) {
+			# Perhaps plan does not exist? Prepare/compile plan.
 				$params = '';
 				foreach($inputarr as $v) {
 					if ($params) $params .= ',';
@@ -607,40 +666,26 @@ WHERE c2.relname=\'%s\'';
 						$params .= "REAL";
 					}
 				}
-				$sqlarr = explode('?',$sqltxt);
-				$sqltxt = '';
+				$sqlarr = explode('?',$sql);
+				//print_r($sqlarr);
+				$sql = '';
 				$i = 1;
 				foreach($sqlarr as $v) {
-					$sqltxt .= $v.'$'.$i;
+					$sql .= $v.' $'.$i;
 					$i++;
 				}
-				$s = "PREPARE $plan ($params) AS ".substr($sqltxt,0,strlen($sqltxt)-2);		
-				adodb_pr($s);
+				$s = "PREPARE $plan ($params) AS ".substr($sql,0,strlen($sql)-2);		
+				//adodb_pr($s);
 				pg_exec($this->_connectionID,$s);
 				echo $this->ErrorMsg();
-			} else {
-				$plan = $sql[1];
-			}
-			$params = '';
-			foreach($inputarr as $v) {
-				if ($params) $params .= ',';
-				if (is_string($v)) {
-					if (strncmp($v,"'",1) !== 0) $params .= $this->qstr($v.'TEST');
-				} else {
-					$params .= $v;
-				}
 			}
 			
-			if ($params) $sql = "EXECUTE $plan ($params)";
-			else $sql = "EXECUTE $plan";
-			
-			adodb_pr(">>>>>".$sql);
-			pg_exec($this->_connectionID,$s);
-		}*/
-		
-		$this->_errorMsg = false;
-		
-		$rez = pg_exec($this->_connectionID,$sql);
+			$rez = pg_exec($this->_connectionID,$exsql);
+		} else {
+			$this->_errorMsg = false;
+			//adodb_backtrace();
+			$rez = pg_exec($this->_connectionID,$sql);
+		}
 		// check if no data returned, then no need to create real recordset
 		if ($rez && pg_numfields($rez) <= 0) {
 			if (is_resource($this->_resultid) && get_resource_type($this->_resultid) === 'pgsql result') {
@@ -677,7 +722,10 @@ WHERE c2.relname=\'%s\'';
 	function ErrorNo()
 	{
 		$e = $this->ErrorMsg();
-		return strlen($e) ? $e : 0;
+		if (strlen($e)) {
+			return ADOConnection::MetaError($e);
+		 }
+		 return 0;
 	}
 
 	// returns true or false
@@ -748,14 +796,15 @@ class ADORecordSet_postgres64 extends ADORecordSet{
 	function _initrs()
 	{
 	global $ADODB_COUNTRECS;
-		$this->_numOfRows = ($ADODB_COUNTRECS)? @pg_numrows($this->_queryID):-1;
-		$this->_numOfFields = @pg_numfields($this->_queryID);
+		$qid = $this->_queryID;
+		$this->_numOfRows = ($ADODB_COUNTRECS)? @pg_numrows($qid):-1;
+		$this->_numOfFields = @pg_numfields($qid);
 		
 		// cache types for blob decode check
-		for ($i=0, $max = $this->_numOfFields; $i < $max; $i++) { 
-			$f1 = $this->FetchField($i);
-			//print_r($f1);
-			if ($f1->type == 'bytea') $this->_blobArr[$i] = $f1->name;
+		for ($i=0, $max = $this->_numOfFields; $i < $max; $i++) {  
+			if (pg_fieldtype($qid,$i) == 'bytea') {
+				$this->_blobArr[$i] = pg_fieldname($qid,$off);
+			}
 		}		
 	}
 
@@ -782,8 +831,6 @@ class ADORecordSet_postgres64 extends ADORecordSet{
 		$o->name = @pg_fieldname($this->_queryID,$off);
 		$o->type = @pg_fieldtype($this->_queryID,$off);
 		$o->max_length = @pg_fieldsize($this->_queryID,$off);
-		//print_r($o);		
-		//print "off=$off name=$o->name type=$o->type len=$o->max_length<br>";
 		return $o;	
 	}
 
@@ -819,9 +866,8 @@ class ADORecordSet_postgres64 extends ADORecordSet{
 			$this->_currentRow++;
 			if ($this->_numOfRows < 0 || $this->_numOfRows > $this->_currentRow) {
 				$this->fields = @pg_fetch_array($this->_queryID,$this->_currentRow,$this->fetchMode);
-			
-				if (is_array($this->fields)) {
-					if (isset($this->_blobArr)) $this->_fixblobs();
+				if (is_array($this->fields) && $this->fields) {
+					if ($this->fields && isset($this->_blobArr)) $this->_fixblobs();
 					return true;
 				}
 			}
@@ -833,11 +879,13 @@ class ADORecordSet_postgres64 extends ADORecordSet{
 	
 	function _fetch()
 	{
+				
 		if ($this->_currentRow >= $this->_numOfRows && $this->_numOfRows >= 0)
         	return false;
 
 		$this->fields = @pg_fetch_array($this->_queryID,$this->_currentRow,$this->fetchMode);
-		if (isset($this->_blobArr)) $this->_fixblobs();
+		
+	if ($this->fields && isset($this->_blobArr)) $this->_fixblobs();
 			
 		return (is_array($this->fields));
 	}
